function [U, V] = gpsgridder (xo, yo, xi, yi, ui, vi, dr, nu)
%GPSGRIDDER Thin-plate coupled interpolation of GPS velocity components
%   [U, V] = gpsgridder (x, y, xi, yi, ui, vi, dr, mu)
%  Input:
%   xo, yo: Equidistant lattice for output x and y nodes (column vectors)
%   xi, yi: Locations of data points (column vectors)
%   ui, vi: x- and y-components of GPS velocities (column vectors)
%   dr: Minimum distance between points
%   nu: Poisson's ratio
%  Output:
%   U, V: Grids with east-west and north-south velocities

n = length(xi); n1 = n + 1; n2 = 2*n;
% Remove mean value from each velocity component:
u0 = mean(ui);  ui = ui - u0; 
v0 = mean(vi);  vi = vi - v0;
% Evaluate the three Greens functions q, p, w:
[q, p, w] = get_qpw ([xi yi], [xi yi], dr, nu);
% Solve for the weights wt:
wt = [q w; w p] \ [ui; vi];
%[U,S,V] = svd ([q w; w p]);
%s = diag(S);
%s(1:2400) = 1.0 / s(1:2400); s(2401:n2) = 0;
%wt = V*diag(s)*(U'*[ui; vi]);
% Create the 2-D mesh grid based on output xo,yo vectors
[X, Y] = meshgrid (xo, yo);
[nrows, ncols] = size (X);
% Evaluate solution for U,V:
U = zeros (size(X));    V = zeros (size(X));
for row = 1:nrows
    for col = 1:ncols;
        [q, p, w] = get_qpw ([xi yi], [X(row,col) Y(row,col)], dr, nu);
        U(row,col) = u0 + q * wt(1:n) + w * wt(n1:n2);
        V(row,col) = v0 + w * wt(1:n) + p * wt(n1:n2);
    end
end
end

function [r,dx,dy] = get_radius (X, P)
% GET_RADIUS  Returns dx, dy,and radial distances between points in X and P
%   [r,dx,dy] = get_radius (X, P)
%     X: rows contain the 2-dimensional coordinates in X
%     P: rows contain the 2-dimensional coordinates in P
[n, ~] = size (X); % n is number of points in X
[m, ~] = size (P); % m is number of points in P
dx=zeros(m,n);  dy=zeros(m,n);
for k=1:m
    dx(k,:) = (X(:,1) - ones(n,1)*P(k,1))';    % dx
    dy(k,:) = (X(:,2) - ones(n,1)*P(k,2))';    % dy
end
r = hypot (dx, dy);
end

function [q, p, w] = get_qpw (X, P, dr, nu)
% Compute the Green's functions for all (X;P) combinations
[r,dx,dy] = get_radius (X, P);
logr = log (r+dr);  % Add fudge dr term
r2 = (r+dr).^2;
q = (3 - nu) * logr + (1+nu)*(dy.^2)./dr;
p = (3 - nu) * logr + (1+nu)*(dx.^2)./dr;
w = -(1+nu)*dx.*dy./r2;
end
